@@ -16,7 +16,7 @@
 __author__ = "Anton Panasenko"
 __copyright__ = "Copyright 2013, Qubell.com"
 __license__ = "Apache"
-__version__ = "1.0.9"
+__version__ = "1.0.10"
 __email__ = "apanasenko@qubell.com"
 
 import unittest
@@ -86,20 +86,60 @@ def workflow(name, parameters=None, timeout=10):
     return wrapper
 
 
+def environment(params):
+    def copy(func, name=None):
+        import types
+
+        if not name:
+            name = func.func_name
+
+        return types.FunctionType(func.func_code, func.func_globals, name=name,
+                                  argdefs=func.func_defaults,
+                                  closure=func.func_closure)
+
+    def wraps_class(clazz):
+        clazz.environments = params
+
+        methods = filter(lambda (name, func): name.startswith("test"), clazz.__dict__.items())
+
+        for name_method, _ in methods:
+            delattr(clazz, name_method)
+
+        for name in params.keys():
+            for name_method, method in methods:
+                new_name = name_method + "_on_environment_" + name
+                method = copy(method, new_name)
+                setattr(clazz, new_name, method)
+
+        return clazz
+    return wraps_class
+
+
 # noinspection PyPep8Naming
 def instance(byApplication):
     def wrapper(func):
+        def get_environment_name(self, f):
+            separator = "_on_environment_"
+            if len(f.__name__.split(separator)) > 1:
+                env = f.__name__.split(separator)[1]
+            elif "_testMethodName" in self.__dict__ and len(self._testMethodName.split(separator)) > 1:
+                env = self._testMethodName.split(separator)[1]
+            else:
+                env = "default"
+            return env
+
         @wraps(func)
         def wrapped_func(*args, **kwargs):
             self = args[0]
+            env = get_environment_name(self, func)
 
-            def findByApplicatioName(app):
-                for instance in self.instances:
-                    if instance.applicationName == app:
-                        return instance
+            def find_by_application_name(app):
+                for inst in self.instances:
+                    if inst.applicationName == app and inst.environmentName == env:
+                        return inst
                 return None
 
-            func(*args + (findByApplicatioName(byApplication),), **kwargs)
+            func(*args + (find_by_application_name(byApplication),), **kwargs)
         return wrapped_func
     return wrapper
 
@@ -108,9 +148,18 @@ class BaseTestCase(unittest.TestCase):
     platform = None
     parameters = None
     sandbox = None
+    environments = None
 
     @classmethod
     def environment(cls, organization):
+        import re
+        if cls.environments:
+            envs = {}
+            for name, value in cls.environments.items():
+                envs.update({str(re.sub("[^a-zA-Z0-9_]", "", name)): value})
+        else:
+            envs = {"default": {}}
+
         return {
             "organization": {"name": organization},
             "services": [
@@ -128,7 +177,7 @@ class BaseTestCase(unittest.TestCase):
                                   "jcloudsCredential": cls.parameters['provider_credential'],
                                   "jcloudsRegions": cls.parameters['provider_region']
             }],
-            "environments": [{"name": "default"}]}
+            "environments": envs}
 
     @classmethod
     def apps(cls):
@@ -153,10 +202,12 @@ class BaseTestCase(unittest.TestCase):
 
         cls.instances = []
         for app in cls.sandbox['applications']:
-            if app.get('launch', True):
-                instance = cls.organization.application(name=app['name']).launch()
-                cls.instances.append(instance)
-                cls.sandbox.sandbox["instances"].append({"id": instance.instanceId, "name": instance.name})
+            for env in cls.sandbox['environments']:
+                if app.get('launch', True):
+                    environment_id = cls.organization.environment(name=env).id
+                    instance = cls.organization.application(name=app['name']).launch(environmentId=environment_id)
+                    cls.instances.append(instance)
+                    cls.sandbox.sandbox["instances"].append({"id": instance.instanceId, "name": instance.name})
 
         for instance in cls.instances:
             if not instance.ready(timeout=timeout):
@@ -216,16 +267,19 @@ class SandBox(object):
     def make(self):
         log.info("preparing sandbox...")
 
-        for env in self.sandbox["environments"]:
-            environment = self.organization.environment(name=env["name"])
+        for name, env in self.sandbox["environments"].items():
+            environment = self.organization.environment(name=name)
             environment.clean()
 
-            for service in self.sandbox["services"]:
+            for service in self.sandbox["services"] + env.get('services', []):
                 self.__service(environment, service)
 
-            for provider in self.sandbox["cloudAccounts"]:
+            for provider in self.sandbox["cloudAccounts"] + env.get('cloudAccounts', []):
                 self.__cloud_account(environment, provider)
 
+            for police in env.get('policies', []):
+                environment.policyAdd(police)
+
             for app in self.sandbox["applications"]:
                 self.__application(app)
 
@@ -244,6 +298,10 @@ class SandBox(object):
                 )
         log.info("sandbox cleaned")
 
+    def __check_environment_name(self, name):
+        import re
+        re.sub("")
+
     def __getitem__(self, name):
         if name in self.sandbox:
             return self.sandbox[name]
